@@ -63,7 +63,9 @@ def cli(ctx: click.Context, verbose: bool = False) -> None:
 
 @cli.command()
 @click.argument(
-    "repository_path", type=click.Path(exists=True, file_okay=False, dir_okay=True)
+    "repository_path",
+    type=click.Path(exists=True, file_okay=False, dir_okay=True),
+    required=True,
 )
 @click.option(
     "--languages",
@@ -129,6 +131,13 @@ def analyze(
         logger = get_logger(__name__)
         verbose = ctx.obj.get("verbose", False)
 
+        # If monorepo mode and .codeql.json exists, default repository_path to current directory
+        root_config_path = Path(repository_path) / ".codeql.json"
+        if monorepo and root_config_path.exists():
+            logger.info(
+                "Detected .codeql.json in root. Using current directory as repository path."
+            )
+
         logger.info(f"Starting CodeQL analysis for: {repository_path}")
 
         # Validate upload-sarif parameters if upload is requested
@@ -461,8 +470,8 @@ def upload_sarif(
         if not final_repository:
             click.echo(
                 click.style("ERROR:", fg="red", bold=True)
-                + " Repository is required. Provide --repository or ensure you're in a Git repository "
-                "with a GitHub remote configured.",
+                + " Repository is required. Provide --repository or ensure you're in a Git "
+                "repository with a GitHub remote configured.",
                 err=True,
             )
             sys.exit(1)
@@ -470,7 +479,8 @@ def upload_sarif(
         if not final_commit_sha:
             click.echo(
                 click.style("ERROR:", fg="red", bold=True)
-                + " Commit SHA is required. Provide --commit-sha or ensure you're in a Git repository.",
+                + " Commit SHA is required. Provide --commit-sha or ensure you're in a Git "
+                "repository.",
                 err=True,
             )
             sys.exit(1)
