@@ -1,6 +1,8 @@
 """CodeQL analysis use case implementation."""
 
 import json
+import os
+from concurrent.futures import ProcessPoolExecutor, as_completed
 from datetime import datetime
 from pathlib import Path
 from typing import Any, List, Optional, Set
@@ -22,6 +24,8 @@ from ...infrastructure.codeql_runner import CodeQLRunner
 class CodeQLAnalysisUseCase:
     """Use case for running CodeQL analysis on repositories."""
 
+    DEFAULT_MAX_WORKERS: int = 10
+
     def __init__(self, logger: Any) -> None:
         """Initialize the use case with dependencies."""
         self._logger = logger
@@ -31,15 +35,35 @@ class CodeQLAnalysisUseCase:
 
     def execute(self, request: CodeQLAnalysisRequest) -> RepositoryAnalysisSummary:
         """
-        Execute CodeQL analysis on a repo or monorepo
+        Execute CodeQL analysis on a repo or monorepo.
+
+        Args:
+            request: CodeQL analysis request
+
+        Returns:
+            RepositoryAnalysisSummary with analysis results
+
+        Raises:
+            ValueError: If request is invalid
+            Exception: If analysis fails
         """
         try:
             if request.monorepo:
-                self._logger.info(
-                    f"Starting monorepo CodeQL analysis for: "
-                    f"{request.repository_path}"
-                )
-                return self._execute_monorepo_analysis(request)
+                # Run scan based on .codeql.json if it exists
+                root_config_path = request.repository_path / ".codeql.json"
+                if root_config_path.exists():
+                    self._logger.info(
+                        "Detected .codeql.json. Using project config mode."
+                    )
+                    with open(root_config_path, "r", encoding="utf-8") as f:
+                        config_data = json.load(f)
+                    return self._execute_monorepo_analysis(request, config_data)
+                else:
+                    self._logger.info(
+                        "Starting monorepo CodeQL analysis for: %s",
+                        request.repository_path,
+                    )
+                    return self._execute_monorepo_analysis(request)
             else:
                 self._logger.info(
                     f"Starting single repository CodeQL analysis for: "
@@ -51,53 +75,144 @@ class CodeQLAnalysisUseCase:
             self._logger.error(f"CodeQL analysis failed: {e}")
             raise
 
-    def _execute_monorepo_analysis(
-        self, request: CodeQLAnalysisRequest
+    def _process_monorepo_project(
+        self, project_path: Path, request: CodeQLAnalysisRequest
     ) -> RepositoryAnalysisSummary:
-        """
-        Execute CodeQL analysis on a monorepo by analyzing each project.
-        """
-        self._logger.info("Starting monorepo analysis...")
+        """Process a single project inside a monorepo."""
+        self._logger.info(f"Processing project: {project_path}")
+        sub_request = CodeQLAnalysisRequest(
+            repository_path=project_path,
+            force_install=request.force_install,
+            target_languages=request.target_languages,
+            verbose=request.verbose,
+            output_directory=request.output_directory,
+        )
+        try:
+            return self._execute_single_repo_analysis(sub_request)
+        except Exception as e:
+            self._logger.exception(f"Analysis failed for {project_path}: {e}")
+            return RepositoryAnalysisSummary(
+                repository_path=project_path,
+                detected_projects=[],
+                analysis_results=[],
+                error=str(e),
+            )
 
-        # Find all 1st-level folders inside the monorepo path
-        sub_project_paths = [p for p in request.repository_path.iterdir() if p.is_dir()]
+    def _execute_monorepo_analysis(
+        self, request: CodeQLAnalysisRequest, config_data: Optional[dict] = None
+    ) -> RepositoryAnalysisSummary:
+        self._logger.info("Starting monorepo analysis (parallelized)...")
 
-        if not sub_project_paths:
-            self._logger.warning("No projects found in the monorepo path.")
+        if config_data:
+            projects_config = config_data.get("projects", [])
+            use_config_mode = True
+        else:
+            use_config_mode = False
+            projects_config = [
+                {"path": str(p), "build-mode": "none"}
+                for p in request.repository_path.iterdir()
+                if p.is_dir() and not p.name.startswith(".")
+            ]
+
+        if not projects_config:
+            self._logger.warning("No projects found.")
             return RepositoryAnalysisSummary(
                 repository_path=request.repository_path,
                 detected_projects=[],
                 analysis_results=[],
+                error="No projects found.",
             )
 
         all_detected_projects = []
         all_analysis_results = []
+        error_messages = []
 
-        for sub_path in sub_project_paths:
-            self._logger.info(f"Processing project: {sub_path}")
+        max_workers = min(os.cpu_count() or 1, self.DEFAULT_MAX_WORKERS)
+        with ProcessPoolExecutor(max_workers=max_workers) as executor:
+            futures = []
+            for project_cfg in projects_config:
+                if use_config_mode:
+                    futures.append(
+                        executor.submit(
+                            self._process_monorepo_project_from_config,
+                            project_cfg,
+                            request,
+                        )
+                    )
+                else:
+                    futures.append(
+                        executor.submit(
+                            self._process_monorepo_project,
+                            Path(project_cfg["path"]),
+                            request,
+                        )
+                    )
 
-            # Create a new request object for the project
-            sub_request = CodeQLAnalysisRequest(
-                repository_path=sub_path,
-                force_install=request.force_install,
-                target_languages=request.target_languages,
-                verbose=request.verbose,
-                output_directory=request.output_directory,
-            )
+            # Collect results from futures
+            for future in as_completed(futures):
+                try:
+                    result = future.result()
+                    all_detected_projects.extend(result.detected_projects)
+                    all_analysis_results.extend(result.analysis_results)
+                    if result.error:
+                        error_messages.append(result.error)
+                except Exception as e:
+                    self._logger.exception(f"Processing failed: {e}")
+                    error_messages.append(str(e))
 
-            try:
-                summary = self._execute_single_repo_analysis(sub_request)
-                all_detected_projects.extend(summary.detected_projects)
-                all_analysis_results.extend(summary.analysis_results)
-            except Exception as e:
-                self._logger.error(f"Analysis failed for {sub_path}: {e}")
+        # Compose aggregated error string, if any
+        aggregated_error = "\n".join(error_messages) if error_messages else None
 
         return RepositoryAnalysisSummary(
             repository_path=request.repository_path,
             detected_projects=all_detected_projects,
             analysis_results=all_analysis_results,
+            error=aggregated_error,
         )
 
+    def _process_monorepo_project_from_config(
+        self, project_cfg: dict, request: CodeQLAnalysisRequest
+    ) -> RepositoryAnalysisSummary:
+        try:
+            project_path = Path(project_cfg["path"])
+            build_mode = project_cfg.get("build-mode", "none")
+            build_script = project_cfg.get("build-script")
+            queries = project_cfg.get("queries", [])
+
+            if build_script:
+                root_path = (
+                    request.repository_path.resolve()
+                )  # Resolve current working directory
+                build_script = str(root_path / build_script)
+                self._logger.debug(f"Resolved build_script path: {build_script}")
+
+            self._logger.info(
+                f"Processing project from config: {project_path} (build-mode: {build_mode})"
+            )
+
+            sub_request = CodeQLAnalysisRequest(
+                repository_path=project_path,
+                force_install=request.force_install,
+                target_languages=request.target_languages,
+                verbose=request.verbose,
+                output_directory=request.output_directory,
+                build_mode=build_mode,
+                build_script=build_script,
+                queries=queries,
+            )
+            return self._execute_single_repo_analysis(sub_request)
+
+        except Exception as e:
+            self._logger.exception(
+                f"Analysis failed for {project_cfg.get('path')}: {e}"
+            )
+            return RepositoryAnalysisSummary(
+                repository_path=Path(project_cfg.get("path", ".")),
+                detected_projects=[],
+                analysis_results=[],
+                error=str(e),
+            )
+
     def _execute_single_repo_analysis(
         self, request: CodeQLAnalysisRequest
     ) -> RepositoryAnalysisSummary:
@@ -339,7 +454,10 @@ class CodeQLAnalysisUseCase:
 
             # Analyze each language in the project
             for language in project.languages:
-                self._logger.debug(f"Running CodeQL analysis for {language.value}")
+                self._logger.debug(
+                    f"Running CodeQL analysis for {language.value} with build_mode={request.build_mode}"
+                    f", build_script={request.build_script}"
+                )
 
                 # Create database and run analysis using the robust method
                 if self._codeql_runner is None:
@@ -362,13 +480,26 @@ class CodeQLAnalysisUseCase:
                 file_extension = format_to_extension.get(output_format, ".sarif")
                 output_file = output_dir / f"results-{language.value}{file_extension}"
 
+                # Explicitly set build_command to None if build_mode is "none"
+                build_command = None
+                if request.build_mode != "none" and request.build_script:
+                    build_command = request.build_script
+                    self._logger.debug(f"Using build command: {build_command}")
+                else:
+                    self._logger.debug(
+                        f"No build command used (build_mode={request.build_mode})"
+                    )
+
                 # Use the robust create_and_analyze method that handles corrupted databases
                 analysis_result = self._codeql_runner.create_and_analyze(
                     source_root=str(project.path),
                     language=language.value,
                     output_file=str(output_file),
                     database_name=str(output_dir / f"db-{language.value}"),
+                    build_command=build_command,  # Use conditional build_command
                     cleanup_database=False,  # Keep database for debugging
+                    build_mode=request.build_mode,  # Pass build_mode
+                    queries=request.queries,
                 )
 
                 if not analysis_result.success:
@@ -447,10 +578,12 @@ class CodeQLAnalysisUseCase:
 
     def _count_sarif_findings(self, sarif_file: Path) -> int:
         """Count findings in a SARIF file."""
-        try:
-            import json
+        if not sarif_file.exists():
+            self._logger.warning(f"SARIF file does not exist: {sarif_file}")
+            return 0
 
-            with open(sarif_file, "r") as f:
+        try:
+            with open(sarif_file, "r", encoding="utf-8") as f:
                 sarif_data = json.load(f)
 
             total_findings = 0
@@ -459,7 +592,13 @@ class CodeQLAnalysisUseCase:
                 total_findings += len(results)
 
             return total_findings
-        except Exception:
+        except (json.JSONDecodeError, OSError) as e:
+            self._logger.warning(f"Failed to parse SARIF file {sarif_file}: {e}")
+            return 0
+        except Exception as e:
+            self._logger.warning(
+                f"Unexpected error reading SARIF file {sarif_file}: {e}"
+            )
             return 0
 
     def _export_codeql_suites_path(self) -> None:
